/*
 * d_arch.c
 *
 * This file is part of the dat library.
 *
 * Copyright 1989 by the Movies of the Future Group
 * of the MIT Media Laboratory
 * All rights reserved.
 *
 * Permission to use, copy, and modify this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of M.I.T. not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  This software or any
 * other copies thereof may not be provided or otherwise made available
 * to others.  No title to or ownership of the software is hereby
 * transferred.  M.I.T. makes no representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express
 * or implied warranty.
 *
 * $Source: /nfs/source/MASTER/garden/datlib/RCS/d_arch.c,v $
 * $Date: 94/01/07 21:57:37 $
 * $Author: sbeck $
 * $Locker:  $
 *
 * $Log:	d_arch.c,v $
 * Revision 1.4  94/01/07  21:57:37  sbeck
 * Created mach_get_byte_order() which determines the
 * endian-ness of the host machine.
 * 
 * Removed all evidence of floating point representation
 * since it is now completely handled by proper byte order
 * processing. Also changed usage of the predefined macro,
 * MACH_BYTE_ORDER to the function call mach_get_byte_order() 
 * which automatically determines the endian-ness of the host
 * machine.
 * 
 * Revision 1.3  93/12/18  15:57:47  sbeck
 * Due to alpha_osf1 incompatibilities changed BYTE_ORDER to BYTE_ORDER_KEY
 * -Shawn
 * 
 * Revision 1.2  1991/11/02  23:31:17  klee
 * updated for const declarations
 *
 * Revision 1.1  90/08/28  14:40:45  pizza
 * Initial revision
 * 
 *
 * Created: Tony Pizzigati 6/17/90
 */

static char rcsid[] = "$Header: /nfs/source/MASTER/garden/datlib/RCS/d_arch.c,v 1.4 94/01/07 21:57:37 sbeck Exp $" ;

#include "dat.h"

/* Global variables - must reflect architecture definitions in dat.h */

struct b_o {
  char			*value;	/* string representing byte order */
  enum byte_order	code;	/* one character keycode for byte order */
};

#define MSB_BO_STRING	"msb"
#define LSB_BO_STRING	"lsb"

static struct b_o bo_list[] = {
  {MSB_BO_STRING,		MSB},
  {LSB_BO_STRING,	        LSB},
  {(char*)NULL,			NOKEY}
} ;

enum byte_order d_get_byte_order(d)
     DATFILE *d;
{
  register int i ;
  const char *p ;

  p = d_getkey(d, BYTE_ORDER_KEY) ;

  if (p == NULL)
    return (NOKEY) ;

  for (i = 0; bo_list[i].value; i++) {
    if(!strcmp(p, bo_list[i].value))
      return(bo_list[i].code);
  }

  return (NOKEY) ;
}

int d_set_byte_order(d, bo)
     DATFILE		*d ;
     enum byte_order	bo ;
{
  register int i ;

  for (i = 0 ; bo_list[i].value ; i++) {
    if(bo_list[i].code == bo) {
      d_setkey(d, BYTE_ORDER_KEY, bo_list[i].value) ;
      return (0) ;
    }
  }
  return (-1) ;
}

/*
** FUNCTION: mach_get_byte_order
**
** SIDE EFFECTS: none.
**
** COMMENTS: returns the machine's byte_order
**	
*/
enum byte_order	mach_get_byte_order()
{
  union {
    unsigned char  b[2];
    unsigned short u;
  } tmp;
  tmp.u = 0xAABB;
  if( (tmp.b[0] == 0xBB) && (tmp.b[1] == 0xAA) )
    return LSB;
  else if( (tmp.b[0] == 0xAA) && (tmp.b[1] == 0xBB) )
    return MSB;
  else {
    fprintf(stdout,"ERROR in d_arch.c: mach_get_byte_order()\n");
    return (enum byte_order)-1;
  }
}


