/*
Copyright 1989 by the Movies of the Future Group
of the MIT Media Laboratory
All rights reserved.

Permission to use, copy, and modify this software and its
documentation for any purpose and without fee is hereby granted,
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in
supporting documentation, and that the name of M.I.T. not be used in
advertising or publicity pertaining to distribution of the software
without specific, written prior permission.  This software or any
other copies thereof may not be provided or otherwise made available to
others.  No title to or ownership of the software is hereby transferred.
M.I.T. makes no representations about the suitability of this software for any
purpose.  It is provided "as is" without express or implied warranty.
*/
/* EMACS_MODES: c */
#include "dat.h"
#include "datio.h"
#include <sys/types.h>

#define IOBSIZE 1024

int 
  _dread(df, buf, n, cvt)
DATFILE *df;
char buf[];
int n, cvt;
{
  register struct dinfo *ip;
  int fd, d, buflbps, iocnt, bytes, iobytes;
  enum byte_order bo;
  int cvt_flag,cve_flag;
  char *iop;

  fd = fileno(d_getfp(df));
  if ((ip=_Dinfo[fd]) == NULLI) 
    {
#ifdef APPEND_MODE
      if (d_submatrix(df, (struct d_addr *)-1, NULLA, NULLA) < 0)
	return(-1);
#else
      if (d_submatrix(df, NULLA, NULLA, NULLA) < 0)
	return(-1);
#endif
      ip = _Dinfo[fd];
    }

  /* cve_flag == 1 means convert endian format */
  bo = d_get_byte_order(df);
  cve_flag = ( (bo!=NOKEY) && (bo!=mach_get_byte_order()) );

  /* cvt == 0 means cvt same data type as datfile */
  /* cvt_flag == 1 means convert data type */
  if( cvt == 0 ) {
    cvt_flag = 0;
    cvt = ip->dtype;
  }
  else
    cvt_flag = (cvt != ip->dtype);

  buflbps = (!cvt_flag) ? ip->lbps : _dsize(cvt);
  iop = buf;
  
  if (n<0) 
    n = ip->subdim[0];		/* negative n means 1 line */
  
  while (n>0 && !ip->eof) 
    {	/* while more elements are wanted */
      iocnt = ip->subdim[0] - ip->tell[0];

      if (iocnt > n) 
	iocnt = n;
 
      if ((ip->hsub == 1) && !cvt_flag && !cve_flag )
	{	/* easy case */
	  bytes = iocnt << ip->lbps;
	  if ((iobytes=read(fd, iop, bytes)) < bytes) 
	    goto error;

	  iop += iobytes;
	}
      else 
	{	/* general case */
	  char iobuf[IOBSIZE];
	  int chunk = (sizeof iobuf) / (ip->hsub << ip->lbps);
	  
	  if (iocnt > chunk) 
	    iocnt = chunk;

	  bytes = ((iocnt-1)*ip->hsub + 1) << ip->lbps;
	  
	  if ((iobytes=read(fd, iobuf, bytes)) < bytes) 
	    goto error;
	  
	  if (ip->hsub > 1) 
	    {
	      lseek(fd, (off_t)(ip->hsub-1)<<ip->lbps, 1);
	      subsample(cvt?iobuf:iop, 
			iobuf, 
			1<<ip->lbps, 
			(int) ip->hsub, 
			iocnt);
	    }

	  /* do endian conversion from file to machine format */
	  if( cve_flag )
	    convert_endian(iobuf, ip->dtype, iocnt);

	  /* now do data type conversion on machine */
	  convert_datatype(iop, cvt, iobuf, ip->dtype, iocnt);

	  iop += iocnt << buflbps;
	}

      n -= iocnt;
      if ((ip->tell[0]+=iocnt) < ip->subdim[0]) 
	continue;
      
      d = 0;		/* adjust ip->tell[] */
      do {
	ip->tell[d] = 0;
	lseek(fd, ip->nskip[d], 1);
	if (++d >= ip->ndims) {
	  ip->eof = 1;
	  break;
	}
      } while (++ip->tell[d] >= ip->subdim[d]);
    }
  return((iop-buf) >> buflbps);
  
 error:
  if (iobytes < 0) 
    {
      perror(d_getname(df));
      return(_daterrh("_dread: read error\n"));
    }
  else
    return(_daterrh("_dread: early EOF on %s\n", d_getname(df)));
}

