/***********************************************************************\
*	audIOsgi.c							*
*	New hardware-indep audio IO interface intended to be portable	*
*	based on lofiplay etc						*
*	dpwe 1993nov17, 13mar92						*
*	SGI AL version - 1994mar30					*
\***********************************************************************/

#include <audio.h>

typedef struct {		/* descriptor for audio IO channel */
    int	mode;			/* AU_INPUT or AU_OUTPUT */
    int chans;			
    int format;
    ALport port;
} AUSTRUCT;

#define AUSTRUCT_DEFINED	/* stop redefinition in audIO.h */

#include "audIO.h"	/* for stdio, PARG etc */

#include <assert.h>

#define DFLT_BUFTIME	(0.25)		/* default buffer length in secs */

#define NUM_PARAMS 1

static int AUSampleSizeOf(int format)
{   /* return number of bytes per sample in given fmt */
    return(format & AU_BYTE_SIZE_MASK);
}	/* copied from SFSampleSizeOf */

AUSTRUCT *AUOpen(int format, double srate, int chans, long desbuf, int mode)
{  /* open a new audio IO channel.  Define data format, #channels and sampling
      rate. Request that output buffer be desbuf frames long.  
      mode is AIO_{INPUT,OUTPUT} */
    AUSTRUCT *au = NULL;
    int	     err = 0;

    DBGFPRINTF((stderr, "AUOpen(fmt %d, sr %.0lf, ch %d, desb %ld, mo %d)\n",
		format, srate, chans, desbuf, mode));
    /* check arguments */
    if(format != AFMT_SHORT && format != AFMT_CHAR && format != AFMT_LONG)  {
	fprintf(stderr, "AUOpen: format %d not supported\n", format);
	++err;
    }
    if(srate < 8000 || srate > 48000 /* || ... */)  {
	fprintf(stderr, "AUOpen: srate %.0f not supported\n", srate);
	++err;
    }
    if(chans < 1 || chans > 2 /* || ... */)  {
	fprintf(stderr, "AUOpen: chans %d not supported\n", chans);
	++err;
    }
    if(mode != AIO_INPUT && mode != AIO_OUTPUT)  {
	fprintf(stderr, "AUOpen: mode %d not supported\n", mode);
	++err;
    }
    /* not sure what to do with desbuf .. */
    if(desbuf <= 0)	
	desbuf = srate * DFLT_BUFTIME;

    if(err == 0)  {	/* actual SGI-specific port initialization */
	ALconfig config;
	long params[2*NUM_PARAMS];
	long width = AUSampleSizeOf(format);

	/* allocate the descriptor */
	au = TMMALLOC(AUSTRUCT, 1, "AUOpen");
	au->mode   = mode;	au->chans = chans;
	au->format = format;	au->port  = NULL;

	/* open the audio */
	config = ALnewconfig();
	ALsetchannels(config, (long)chans);
	ALsetwidth(config, width);
	ALsetqueuesize(config, desbuf);
	DBGFPRINTF((stderr, "AUOpen: desbuf %d\n", desbuf));
	if(au->mode == AIO_INPUT)
	    params[0] = AL_INPUT_RATE;	
	else
	    params[0] = AL_OUTPUT_RATE;	
	params[1] = (long)srate;
	ALsetparams(AL_DEFAULT_DEVICE, params, 2*NUM_PARAMS);
	if(au->mode == AIO_INPUT)
	    au->port = ALopenport("audIOsgi", "r", config);
	else
	    au->port = ALopenport("audIOsgi", "w", config);
	ALfreeconfig(config);
	if(au->port == NULL) {
	    /* error initializing hardware */
	    free(au);
	    return NULL;
	}
    }
    return au;
}

long	AUWrite(AUSTRUCT *au, void *buf, long frames)
{   /* write some sound to the open audio IO channel, per description */
    assert((au->mode) == AIO_OUTPUT);
    ALwritesamps(au->port, buf, frames*au->chans);
    return frames;	/* presumably */
}

long	AURead(AUSTRUCT *au, void *buf, long frames)
{   /* read some sound from the open audio IO channel, per description */
    assert(au->mode == AIO_INPUT);
    ALreadsamps(au->port, buf, frames*au->chans);
    return frames;	/* we presume */
}

void	AUClose(AUSTRUCT *au, int now)
{  /* close an audio IO channel. if <now> is nonzero, stop immediately
      else play out any buffered samples */
    if(now == 0
       && au->mode == AIO_OUTPUT)	/* wait for port to drain on output */
	while(ALgetfilled(au->port)>0)
	    sginap(1);
    ALcloseport(au->port);
    free(au);
}

long AUBufConts(AUSTRUCT *au)
{   /* see how many sample frames are currently in audio buffer.
       (can be used for synchronization, timing etc). */
    return ALgetfilled(au->port);
}
