/***************************************************************\
*   sndfpcm.c			HEADERLESS PCM			*
*   SNDF 'header' functions to access headerless PCM files	*
*   i.e. file is always assumed to be 44k1 stereo 16bit linear  *
*   THIS FILE IS CONDITIONALLY INCLUDED IN sndf.c		*
*   10jul91 after 24sep90 after12aug90 dpwe			*
\***************************************************************/

char *sndfExtn = ".pcm";		/* file extension, if used */

/* header always reads as below.. */
#define DF_PCM_SRT 44100
#define DF_PCM_CHN 2
#define DF_PCM_FMT SFMT_SHORT

#ifdef THINK_C
#include <unix.h>
#else
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <unistd.h>
#endif
#include <fcntl.h>

/* none of this FindFname stuff is needed now we know how to find ANSI lens */
/* private type declarations */
typedef struct fnameinfo
    {
    struct fnameinfo *next;
    FILE   *fhan;
    char   fname[80];
    } FNAMEINFO;	/* used to build a map between FILE * & filenames */

/* static variables */
FNAMEINFO   *fniroot = NULL;	/* root of fnameinfo list */

/* static function prototypes */
#ifdef __STDC__
static char *FindFname(FILE *file);
    /* looks up file ptr, returns mapped name if any */
static size_t GetFileSize(FILE *file);	/* return number of bytes in a file */
/* static size_t fdSize(int fd);	*/
#else
static char *FindFname();
    /* looks up file ptr, returns mapped name if any */
static size_t GetFileSize();	/* return number of bytes in a file */
static size_t fdSize();
#endif

/* ----------------------- implementations ------------------------ */

static void AddFname(fname,file)
    char *fname;
    FILE *file;
    /* associates name 'fname\0' with file ptr 'file' in our lookup table */
    {
    FNAMEINFO *runner = fniroot;

    /* see if FILE is already in our table.. */
    while(runner != NULL && runner->fhan != file)
	runner = runner->next;
    if(runner == NULL)	    /* not found : add new record */
	{
	runner = (FNAMEINFO *)malloc((size_t)sizeof(FNAMEINFO));
	runner->fhan = file;
	runner->next = fniroot;
	fniroot = runner;	/* add to top of list, faster to find */
	}   /* either way, copy in the (new) name */
    strcpy((char *)runner->fname, fname);
    }

static void RmvFname(file)
    FILE *file;
    /* removes any fnameinfo record associated with 'file' */
    {
    FNAMEINFO *node;
    FNAMEINFO **prunner = &fniroot;

    while((*prunner) != NULL && (*prunner)->fhan != file)
	prunner = &(*prunner)->next;
    if((*prunner) != NULL)	/* found : delete record */
	{
	node = (*prunner);
	*prunner = (*prunner)->next;	/* close gap.. */
	free(node);			/* & dispose of memory */
	}
    /* else not found - do nothing */
    }

static char *FindFname(file)
    FILE *file;
    /* looks up file ptr, returns mapped name if any */
    {
    FNAMEINFO *runner = fniroot;

    while(runner != NULL && runner->fhan != file)
	runner = runner->next;
    if(runner != NULL)
	return( (char *)runner->fname );
    else
	return( NULL );     /* fileptr not in our list */
    }

int SFReadHdr(file, snd, infoBmax)
    FILE *file;
    SFSTRUCT *snd;
    int infoBmax;
/* utility to sample the header of an opened file, at most infoBmax info bytes */
    {
    long    infoBsize, infoXtra, bufXtra;
    long    rdsize;
    size_t  num,red;
    long    ft;
    char    *irc_infp, *sdf_infp;
    int     chans,formt,i;
    float   srate;

    if(infoBmax < SFDFLTBYTS)	infoBmax = SFDFLTBYTS;
    infoBmax = (infoBmax)&-4;	/* round DOWN to x4 (can't exceed given bfr) */
    bufXtra = (long)(infoBmax-SFDFLTBYTS);

    ft = ftell(file);
    if( ft > 0 )			/* don't try seek on stdin */
	{
	if(fseek(file,(long)0,SEEK_SET) != 0 ) /* move to start of file */
	    return(SFerror = SFE_RDERR);
	}
    /* Must be ok.  Fabricate sfheader info */
    srate = DF_PCM_SRT;
    chans = DF_PCM_CHN;
    formt = DF_PCM_FMT;
    /* rewrite into different data structure */
    snd->magic = SFMAGIC;
    snd->headBsize = sizeof(SFSTRUCT);
    snd->dataBsize = GetFileSize(file)-4/* bizarre non header */;
    snd->format = formt;
    snd->channels = chans;
    snd->samplingRate = srate;
    /* grab the bizzarre pad samples */
    fread(snd->info, sizeof(short), 2, file);

    return(SFerror = SFE_OK);
    }

int SFRdXInfo(file, psnd, done)
    FILE *file;
    SFSTRUCT *psnd;
    int done;
/* fetch any extra info bytes needed in view of read header
    Passes open file handle, existing sound struct
    already read 'done' bytes; rest inferred from structure */
    {
    size_t  num;
    long    subhlen;
    long    file_pos;
    int     rem;

    /* will not occur for PCM files */
    return(SFerror = SFE_OK);
    }

int SFWriteHdr(file, snd, infoBmax)
    FILE *file;
    SFSTRUCT *snd;
    int infoBmax;
/*  Utility to write out header to open file
    infoBmax optionally limits the number of info chars to write.
    infoBmax = 0 => write out all bytes implied by .headBsize field
    infoBmax >= SFDFLTBYS => write out only infoBmax infobytes */
    {
    size_t  num;
    long    hedSiz;
    char    *irc_infp, *sdf_infp;
    int     i;
    long    infoBsize,ft;
    short   yuk[2];

    if(snd->magic != SFMAGIC)
	return(SFerror = SFE_NSF);
    if(infoBmax < SFDFLTBYTS || infoBmax > SFInfoSize(snd) )
	infoBmax = SFInfoSize(snd);
    ft = ftell(file);
    if( ft > 0 )			/* don't try seek on pipe */
	{
	if(fseek(file, (long)0, SEEK_SET) != 0 )   /* move to start of file */
	return(SFerror = SFE_WRERR);
	}
    /* write nothing .. */
    /* actually, if I write nothing, I get a bus error when I try to 
       write 32768*2 shorts as the first thing in the file.  I can 
       fix it with this one sample pad (YUKYUKYUK) */
    yuk[0] = yuk[1] = 0;
    fwrite(yuk, sizeof(short), 2, file);
    /* perhaps I should skip over this on reads etc .. */
    return(SFerror = SFE_OK);
    }

static long SFHeDaLen(file)
    FILE *file;
    /* return data space used for header in file - see SFCloseWrHdr */
    {
    long hlen = 0;

    /* no header, actually - just two bizarre pad samples */
    return(4L);
    }

static long SFLastSampTell(file)	/* return last valid ftell for smps */
    FILE	*file;
    {
    return (0L);	/* punt on this for here - just use EOF */
    }

static size_t GetFileSize(file)
    FILE *file;	    /* return number of bytes in a file */
    {
    long	fpos, flen;

    fpos = ftell(file);
    if(fseek(file, 0L, SEEK_END)!= -1 && (flen = ftell(file))>0 )
	fseek(file, fpos, SEEK_SET);	/* back where we started */
    else
	flen = SF_UNK_LEN;		/* couldn't establish length */
    return (size_t)flen;
    }

static void FixupSamples(ptr, format, nitems, flag)
    void  *ptr;
    int   format;
    long  nitems;
    int   flag;		/* READ_FLAG or WRITE_FLAG indicates direction */
    {
    }	/* always machine order */
